/***************************************************************************
 *   Copyright (C) 2005-2006 by Georg Hennig                               *
 *   Email: georg.hennig@web.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KOMPARATORWIDGET_H_
#define _KOMPARATORWIDGET_H_

#include <kio/jobclasses.h>

#include "komparatorwidgetbase.h"

class KompareJob;
class KFileItem;
class KFileItemExt;

enum STATUS
{
  CLEAN = 0,
  SEARCHING_URL_1,
  SEARCHING_URL_2,
  COMPARING,
  READY
};

class KQuery;
class KPopupMenu;

class KomparatorWidget : public KomparatorWidgetBase
{
    Q_OBJECT

public:
    KomparatorWidget( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~KomparatorWidget();

    void keyPressEvent( QKeyEvent *event );

    void customEvent( QCustomEvent *event );

    KListView *currentListView; // public for key event handler.

    KompareJob *kompare;        // public for byte comparison (KFileItemExt::getFile(...))

public slots:
    // USER INTERFACE SLOTS
    virtual void slotSearch();
    virtual void slotAbout();
    virtual void slotKListViewDropped( KListView *list_view, QDropEvent *, QListViewItem * );
    virtual void slotContextMenu( KListView *list_view, QListViewItem *item, const QPoint &p );
    virtual void slotRemoveFromList();
    virtual void slotCopyToOtherSide();
    virtual void slotOpenBinding();
    virtual void slotOpenFolder();
    virtual void slotCopySelection();
    virtual void slotTrashFiles();
    virtual void slotDeleteFiles();
    virtual void slotTrashed( KIO::Job *job );
    virtual void slotOpenWith();
    virtual void slotFileProperties();
    virtual void slotNewerR2L();
    virtual void slotNewerL2R();
    virtual void slotMissingR2L();
    virtual void slotMissingL2R();
    virtual void slotTrashDuplicates();
    virtual void slotDeleteDuplicates();
    virtual void slotRemoveSelectionMissing1();
    virtual void slotRemoveSelectionMissing2();
    virtual void slotRemoveSelectionNewer1();
    virtual void slotRemoveSelectionNewer2();
    virtual void slotReferenceLeft( bool is_ref );
    virtual void slotDuplicateSelected();

    // KQUERY SLOTS
    virtual void slotResult( int errorCode, QString errorString );
    virtual void slotAddFile( const KFileItem* item, const QString & );

    // KOMPAREJOB SLOTS
    virtual void slotFileDownloaded( KIO::Job *job );

    // INTERNAL SLOTS
    virtual void slotSizeTypeChanged( int i );
    virtual void slotMD5ChecksumEnabled( bool e );
    virtual void slotEmptyFolder( KIO::Job *job, const KIO::UDSEntryList &list );
    virtual void slotEmptyFolderFinished( KIO::Job *job );
    virtual void slotStatus( QString str );
    virtual void slotProgress( QString str, int percent );
    virtual void slotSaveListViewColumnWidth();

signals:
    void signalStatus( QString );
    void signalProgress( QString, int );
    void signalSaveListViewColumnWidth();

private:
    KQuery *query;
    STATUS status;

    KPopupMenu *popupMenu;

    KFileItemExt *files_dupes;
    KFileItemExt *files_comp;
    KFileItemExt *dirs_dupes;

    uint unreadable_files;
    uint number_of_files;

    int duplicates1_width[4];
    int duplicates2_width[5];
    int missing1_width[4];
    int missing2_width[4];
    int newer1_width[4];
    int newer2_width[4];

    void removeFromLists( KFileItemExt *item );
    void checkEmptyDirs( KURL::List &urls );

    bool initializeQuery();
    bool isDateValid();
    void enable( bool e );
    void resizeListViewColumns();

    KFileItemExt *requested_download;
    KIO::Job *requested_download_job;
};

#endif

